Package: flowGate
Type: Package
Title: Interactive Cytometry Gating in R
Version: 1.12.1
Authors@R: c(person("Andrew", 
                    "Wight", 
                    email = "andrew.wight10@gmail.com", 
                    role = c("aut", "cre")),
             person("Harvey",
                    "Cantor",
                    email = "Harvey_Cantor@dfci.harvard.edu",
                    role = c("aut", "ldr")))
Description: flowGate adds an interactive Shiny app to allow manual 
    GUI-based gating of flow cytometry data in R. Using flowGate, 
    you can draw 1D and 2D span/rectangle gates, quadrant gates, 
    and polygon gates on flow cytometry data by interactively drawing 
    the gates on a plot of your data, rather than by specifying gate 
    coordinates. This package is especially geared toward wet-lab 
    cytometerists looking to take advantage of R for cytometry 
    analysis, without necessarily having a lot of R experience.
License: MIT + file LICENSE
Encoding: UTF-8
Imports: shiny (>= 1.5.0), BiocManager (>= 1.30.10), flowCore (>=
        2.0.1), dplyr (>= 1.0.0), ggplot2 (>= 3.3.2), rlang (>= 0.4.7),
        purrr, tibble, methods
Depends: flowWorkspace (>= 4.0.6), ggcyto (>= 1.16.0), R (>= 4.2)
RoxygenNote: 7.2.3
Suggests: knitr, rmarkdown, stringr, tidyverse, testthat
VignetteBuilder: knitr
biocViews: Software, WorkflowStep, FlowCytometry, Preprocessing,
        ImmunoOncology, DataImport
git_url: https://git.bioconductor.org/packages/flowGate
git_branch: RELEASE_3_23
git_last_commit: fb67327
git_last_commit_date: 2026-07-02
Repository: Bioconductor 3.23
Date/Publication: 2026-07-05
NeedsCompilation: no
Packaged: 2026-07-05 21:46:12 UTC; biocbuild
Author: Andrew Wight [aut, cre],
  Harvey Cantor [aut, ldr]
Maintainer: Andrew Wight <andrew.wight10@gmail.com>
