# Generated by roxygen2: do not edit by hand

export(gs_apply_gating_strategy)
export(gs_gate_interactive)
export(gs_gate_transform_interactive)
import(BiocManager)
import(flowWorkspace)
import(ggcyto)
importFrom(ggplot2,aes)
importFrom(ggplot2,aes_)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_hex)
importFrom(ggplot2,geom_path)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(methods,is)
importFrom(rlang,.data)
importFrom(shiny,reactive)
importFrom(shiny,updateTabsetPanel)
importFrom(tibble,tribble)
