The example flow data used in this package comes from the GvHD data included in
flowCore, the base package for flow cytometry data analysis in R. The GvHD
data that comes with flowCore is already stored in a flowSet, so the first three
samples from that flowSet have been extracted and turned back into .FCS files.
This is to help newcomers understand how to read .FCS/.fcs data into R from raw
data. Please refer to the flowCore package for more information about the
background of these data.